@@ -17,10 +17,6 @@
  * Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #include "thunar/thunar-abstract-dialog.h"
 #include "thunar/thunar-column-editor.h"
 #include "thunar/thunar-dialogs.h"
@@ -325,7 +321,7 @@ thunar_column_editor_init (ThunarColumnEditor *column_editor)
   gtk_widget_show (grid);
 
   /* explain what it does */
-  label = gtk_label_new (_("Show number of containing items"));
+  label = gtk_label_new (_("Show number of contained items:"));
   gtk_label_set_xalign (GTK_LABEL (label), 0.0f);
   gtk_grid_attach (GTK_GRID (grid), label, 0, 0, 1, 1);
   gtk_widget_show (label);
