@@ -17,10 +17,6 @@
  * Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #ifdef HAVE_MEMORY_H
 #include <memory.h>
 #endif
@@ -555,7 +551,7 @@ thunar_emblem_chooser_create_button (ThunarEmblemChooser *chooser,
 
   /* lookup the icon info for the emblem */
   scale_factor = gtk_widget_get_scale_factor (GTK_WIDGET (chooser));
-  info = gtk_icon_theme_lookup_icon_for_scale (chooser->icon_theme, emblem, 48, scale_factor, GTK_ICON_LOOKUP_FORCE_SIZE);
+  info = gtk_icon_theme_lookup_icon_for_scale (chooser->icon_theme, emblem, 32, scale_factor, GTK_ICON_LOOKUP_FORCE_SIZE);
   if (G_UNLIKELY (info == NULL))
     return NULL;
 
