@@ -18,10 +18,6 @@
  * Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #include "thunar/thunar-folder.h"
 #include "thunar/thunar-gobject-extensions.h"
 #include "thunar/thunar-io-jobs.h"
@@ -403,7 +399,10 @@ thunar_folder_finalize (GObject *object)
     g_source_remove (folder->files_update_timeout_source_id);
 
   if (folder->monitor != NULL)
-    g_signal_handlers_disconnect_by_data (folder->monitor, folder);
+    {
+      g_signal_handlers_disconnect_by_data (folder->monitor, folder);
+      g_object_unref (folder->monitor);
+    }
 
   if (folder->corresponding_file)
     {
