@@ -190,9 +190,91 @@ thunar_g_file_new_for_bookmarks (void)
 
 
 
+/* internal version, for recursive execution. Uses a lookup table to detect loops */
+static GFile *
+_thunar_g_file_resolve_symlink_internal (GFile *file, GHashTable *lookup_table)
+{
+  g_autoptr (GHashTable) lookup_history = NULL;
+  g_autoptr (GFileInfo) info = NULL;
+
+  _thunar_return_val_if_fail (G_IS_FILE (file), NULL);
+
+  if (lookup_table == NULL)
+    lookup_history = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
+  else
+    lookup_history = g_hash_table_ref (lookup_table);
+
+  /* Resolve the link target recursively, until the target we found is not a symlink */
+  char *file_uri = g_file_get_uri (file);
+
+  if (file_uri == NULL)
+    return NULL;
+
+  /* Detect loops: if the file already was seen, return NULL */
+  if (g_hash_table_contains (lookup_history, file_uri))
+    {
+      g_info ("Symlink loop detected - resolve symlink aborted");
+      g_free (file_uri);
+      return NULL;
+    }
+  g_hash_table_add (lookup_history, file_uri);
+
+  info = g_file_query_info (file,
+                            G_FILE_ATTRIBUTE_STANDARD_SYMLINK_TARGET "," G_FILE_ATTRIBUTE_STANDARD_IS_SYMLINK,
+                            G_FILE_QUERY_INFO_NONE, NULL, NULL);
+  if (info == NULL)
+    return NULL;
+
+  /* Not a symlink: return that file */
+  if (!g_file_info_get_attribute_boolean (info, G_FILE_ATTRIBUTE_STANDARD_IS_SYMLINK))
+    return g_object_ref (file);
+
+  /* It's a symlink, get the target */
+  const char *target_path = g_file_info_get_attribute_byte_string (info, G_FILE_ATTRIBUTE_STANDARD_SYMLINK_TARGET);
+  if (target_path == NULL)
+    {
+      g_info ("Failed to read target path from symlink: %s", g_file_get_path (file));
+      return NULL;
+    }
+
+  /* The target is as well a symlink ... so lets use recursion */
+  if (g_path_is_absolute (target_path))
+    {
+      g_autoptr (GFile) target = g_file_new_for_path (target_path);
+      return _thunar_g_file_resolve_symlink_internal (target, lookup_history);
+    }
+  else
+    {
+      /* Resolve a possible relative path */
+      g_autoptr (GFile) target = NULL;
+      g_autoptr (GFile) parent = NULL;
+      g_autoptr (GFile) parent_resolved = NULL;
+
+      parent = g_file_get_parent (file);
+      if (parent == NULL)
+        return NULL;
+
+      /* Resolve parent, if required */
+      parent_resolved = _thunar_g_file_resolve_symlink_internal (parent, lookup_history);
+
+      if (parent_resolved == NULL)
+        {
+          g_info ("Failed to resolve parent of symlink");
+          return NULL;
+        }
+
+      target = g_file_resolve_relative_path (parent_resolved, target_path);
+
+      /* Sicne as well that file might be a symlink, we need to use recursion */
+      return _thunar_g_file_resolve_symlink_internal (target, lookup_history);
+    }
+}
+
+
+
 /**
  * thunar_g_file_resolve_symlink:
- * @file : a #GFile.
+ * @file : a #GFile
  *
  * Returns the resolved symlink target of @file as a new #GFile.
  * If @file is not a symlink, @file will just be returned
@@ -202,27 +284,7 @@ thunar_g_file_new_for_bookmarks (void)
 GFile *
 thunar_g_file_resolve_symlink (GFile *file)
 {
-  gchar *basename;
-  GFile *parent = NULL;
-  GFile *target = NULL;
-
-  _thunar_return_val_if_fail (G_IS_FILE (file), NULL);
-
-  parent = g_file_get_parent (file);
-  if (parent == NULL)
-    return NULL;
-
-  basename = g_file_get_basename (file);
-  if (basename == NULL)
-    {
-      g_object_unref (parent);
-      return NULL;
-    }
-
-  target = g_file_resolve_relative_path (parent, basename);
-  g_object_unref (parent);
-  g_free (basename);
-  return target;
+  return _thunar_g_file_resolve_symlink_internal (file, NULL);
 }
 
 
@@ -1756,8 +1818,10 @@ thunar_g_file_get_content_type (GFile *gfile)
           g_object_unref (info);
           info = NULL;
           if (link_target != NULL)
-            info = thunar_g_file_get_content_type_query_info (link_target, err);
-          g_object_unref (link_target);
+            {
+              info = thunar_g_file_get_content_type_query_info (link_target, err);
+              g_object_unref (link_target);
+            }
         }
     }
 
@@ -2074,3 +2138,111 @@ thunar_g_update_user_special_dir (GFile       *file,
   g_free (command);
   g_free (path);
 }
+
+
+
+/**
+ * thunar_g_file_fs_uses_fat_name_scheme:
+ * @file     : a #GFile.
+ *
+ * Returns TRUE, if the filesystem of this file requires usage of FAT naming scheme, FALSE otherwise
+ */
+gboolean
+thunar_g_file_fs_uses_fat_name_scheme (GFile *gfile)
+{
+  gboolean use_fat_name_scheme = FALSE; /* default */
+  g_autoptr (GFile) parent = NULL;
+  g_autoptr (GFileInfo) info = NULL;
+
+  _thunar_return_val_if_fail (G_IS_FILE (gfile), FALSE);
+
+  parent = g_file_get_parent (gfile);
+  if (parent == NULL)
+    {
+      g_warning ("Failed to get parent of %s", g_file_get_uri (gfile));
+      return FALSE;
+    }
+
+  info = g_file_query_filesystem_info (parent,
+                                       G_FILE_ATTRIBUTE_FILESYSTEM_TYPE,
+                                       NULL, NULL);
+  if (info != NULL)
+    {
+      const char *fs_type = g_file_info_get_attribute_string (info, G_FILE_ATTRIBUTE_FILESYSTEM_TYPE);
+
+      /* clang-format off */
+      use_fat_name_scheme =
+        !g_strcmp0 (fs_type, "fat")   ||
+        !g_strcmp0 (fs_type, "vfat")  ||
+        !g_strcmp0 (fs_type, "exfat") ||
+        !g_strcmp0 (fs_type, "fuse")  ||
+        !g_strcmp0 (fs_type, "ntfs")  ||
+        !g_strcmp0 (fs_type, "msdos") ||
+        !g_strcmp0 (fs_type, "msdosfs");
+      /* clang-format on */
+    }
+
+  return use_fat_name_scheme;
+}
+
+
+
+/**
+ * thunar_g_file_transform_to_fat_name_scheme:
+ * @file     : a #GFile.
+ *
+ * Creates a new GFile which has the same URI as @file, but a basename which fulfills the FAT32 naming scheme.
+ * The returned file neds to be released with g_object_unref after usage.
+ *
+ * Return value: (transfer full): A new, possibly renamed GFile, which fulfills the fat naming scheme, or %NULL
+ */
+GFile *
+thunar_g_file_transform_to_fat_name_scheme (GFile *gfile)
+{
+  static const GRegex *windows_reserved_name = NULL;
+  g_autoptr (GFile) parent = NULL;
+  g_autofree gchar *base_name = NULL;
+
+  _thunar_return_val_if_fail (G_IS_FILE (gfile), NULL);
+
+  /* if regex pattern is not initialized, do it */
+  if (G_UNLIKELY (windows_reserved_name == NULL))
+    {
+      /* COM#, LPT#, CON, PRN, AUX, and NUL are not allowed */
+      /* FAT is case-insensitive by default */
+      windows_reserved_name = g_regex_new ("^((COM\\d)|(LPT\\d)|(CON)|(PRN)|(AUX)|(NUL))(\\..*)?$", G_REGEX_CASELESS, 0, NULL);
+    }
+
+  parent = g_file_get_parent (gfile);
+
+  if (parent == NULL)
+    return NULL;
+
+  base_name = g_file_get_basename (gfile);
+
+  /* replace character which are invalid for FAT filenames */
+  g_strdelimit (g_strchomp (base_name), "/:*?\"<>\\|", '_');
+
+  /* ASCII characters 0~31 are as well invalid for FAT filenames */
+  for (int i = 0; base_name[i] != '\0'; i++)
+    {
+      if (base_name[i] >= 0 && base_name[i] < 32)
+        base_name[i] = '_';
+    }
+
+  /* avoid FAT reserved names */
+  if (g_regex_match (windows_reserved_name, base_name, 0, NULL))
+    {
+      g_autofree gchar *tmp = base_name;
+      base_name = g_strconcat ("__", tmp, NULL);
+    }
+
+  /* avoid filename that ends with '.' */
+  if (g_str_has_suffix (base_name, "."))
+    {
+      g_autofree gchar *tmp = base_name;
+      base_name = g_strconcat (tmp, "___", NULL);
+    }
+
+  return g_file_get_child (parent, base_name);
+}
