@@ -17,10 +17,6 @@
  * Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #include "thunar/thunar-application.h"
 #include "thunar/thunar-compact-view.h"
 #include "thunar/thunar-details-view.h"
@@ -636,6 +632,7 @@ thunar_preferences_dialog_init (ThunarPreferencesDialog *dialog)
   gtk_widget_show (grid);
 
   label = gtk_label_new_with_mnemonic (_("_Icon Size:"));
+  gtk_label_set_xalign (GTK_LABEL (label), 0.0);
   gtk_grid_attach (GTK_GRID (grid), label, 0, row, 1, 1);
   gtk_widget_show (label);
 
@@ -669,7 +666,7 @@ thunar_preferences_dialog_init (ThunarPreferencesDialog *dialog)
   /* next row */
   row++;
 
-  button = gtk_check_button_new_with_mnemonic (_("Show Icon _Emblems"));
+  button = gtk_check_button_new_with_mnemonic (_("Show icon _emblems"));
   g_object_bind_property (G_OBJECT (dialog->preferences),
                           "shortcuts-icon-emblems",
                           G_OBJECT (button),
@@ -681,6 +678,112 @@ thunar_preferences_dialog_init (ThunarPreferencesDialog *dialog)
   gtk_grid_attach (GTK_GRID (grid), button, 0, row, 2, 1);
   gtk_widget_show (button);
 
+  /* next row */
+  row++;
+
+  button = gtk_check_button_new_with_mnemonic (_("Show disk space _usage bar"));
+  g_object_bind_property (G_OBJECT (dialog->preferences),
+                          "shortcuts-disk-space-usage-bar",
+                          G_OBJECT (button),
+                          "active",
+                          G_BINDING_BIDIRECTIONAL | G_BINDING_SYNC_CREATE);
+  gtk_widget_set_hexpand (button, TRUE);
+  gtk_grid_attach (GTK_GRID (grid), button, 0, row, 2, 1);
+  gtk_widget_show (button);
+
+  /* next row */
+  row++;
+
+  label = gtk_label_new_with_mnemonic (_("Use warning color when used disk space exceeds:"));
+  g_object_bind_property (G_OBJECT (dialog->preferences),
+                          "shortcuts-disk-space-usage-bar",
+                          G_OBJECT (label),
+                          "sensitive",
+                          G_BINDING_SYNC_CREATE);
+  gtk_widget_set_margin_start (GTK_WIDGET (label), 24);
+  gtk_label_set_xalign (GTK_LABEL (label), 0.0f);
+  gtk_grid_attach (GTK_GRID (grid), label, 0, row, 1, 1);
+  gtk_widget_show (label);
+
+  hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 5);
+  gtk_widget_show (hbox);
+  gtk_grid_attach (GTK_GRID (grid), hbox, 1, row, 1, 1);
+
+  button = gtk_spin_button_new_with_range (0, 100, 1);
+  g_object_bind_property (G_OBJECT (dialog->preferences),
+                          "shortcuts-disk-space-usage-bar",
+                          G_OBJECT (button),
+                          "sensitive",
+                          G_BINDING_SYNC_CREATE);
+  g_object_bind_property (G_OBJECT (dialog->preferences),
+                          "shortcuts-disk-space-usage-warning-percent",
+                          G_OBJECT (button),
+                          "value",
+                          G_BINDING_BIDIRECTIONAL | G_BINDING_SYNC_CREATE);
+  gtk_widget_set_tooltip_text (button, _("Set the value to 0 to disable progress bar color changing"));
+  thunar_gtk_label_set_a11y_relation (GTK_LABEL (label), button);
+  gtk_label_set_mnemonic_widget (GTK_LABEL (label), button);
+  gtk_widget_set_halign (button, GTK_ALIGN_START);
+  gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
+  gtk_widget_show (button);
+
+  label = gtk_label_new ("%");
+  g_object_bind_property (G_OBJECT (dialog->preferences),
+                          "shortcuts-disk-space-usage-bar",
+                          G_OBJECT (label),
+                          "sensitive",
+                          G_BINDING_SYNC_CREATE);
+  gtk_label_set_xalign (GTK_LABEL (label), 0.0f);
+  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+  gtk_widget_show (label);
+
+  /* next row */
+  row++;
+
+  label = gtk_label_new_with_mnemonic (_("Use error color when used disk space exceeds:"));
+  g_object_bind_property (G_OBJECT (dialog->preferences),
+                          "shortcuts-disk-space-usage-bar",
+                          G_OBJECT (label),
+                          "sensitive",
+                          G_BINDING_SYNC_CREATE);
+  gtk_widget_set_margin_start (GTK_WIDGET (label), 24);
+  gtk_label_set_xalign (GTK_LABEL (label), 0.0f);
+  gtk_grid_attach (GTK_GRID (grid), label, 0, row, 1, 1);
+  gtk_widget_show (label);
+
+  hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 5);
+  gtk_widget_show (hbox);
+  gtk_grid_attach (GTK_GRID (grid), hbox, 1, row, 1, 1);
+
+  button = gtk_spin_button_new_with_range (0, 100, 1);
+  g_object_bind_property (G_OBJECT (dialog->preferences),
+                          "shortcuts-disk-space-usage-bar",
+                          G_OBJECT (button),
+                          "sensitive",
+                          G_BINDING_SYNC_CREATE);
+  g_object_bind_property (G_OBJECT (dialog->preferences),
+                          "shortcuts-disk-space-usage-error-percent",
+                          G_OBJECT (button),
+                          "value",
+                          G_BINDING_BIDIRECTIONAL | G_BINDING_SYNC_CREATE);
+  gtk_widget_set_tooltip_text (button, _("Set the value to 0 to disable progress bar color changing"));
+  thunar_gtk_label_set_a11y_relation (GTK_LABEL (label), button);
+  gtk_label_set_mnemonic_widget (GTK_LABEL (label), button);
+  gtk_widget_set_halign (button, GTK_ALIGN_START);
+  gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
+  gtk_widget_show (button);
+
+  label = gtk_label_new ("%");
+  g_object_bind_property (G_OBJECT (dialog->preferences),
+                          "shortcuts-disk-space-usage-bar",
+                          G_OBJECT (label),
+                          "sensitive",
+                          G_BINDING_SYNC_CREATE);
+  gtk_label_set_xalign (GTK_LABEL (label), 0.0f);
+  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+  gtk_widget_show (label);
+
+  /* Tree */
   frame = g_object_new (GTK_TYPE_FRAME, "border-width", 0, "shadow-type", GTK_SHADOW_NONE, NULL);
   gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, TRUE, 0);
   gtk_widget_show (frame);
@@ -735,7 +838,7 @@ thunar_preferences_dialog_init (ThunarPreferencesDialog *dialog)
   /* next row */
   row++;
 
-  button = gtk_check_button_new_with_mnemonic (_("Show Icon E_mblems"));
+  button = gtk_check_button_new_with_mnemonic (_("Show icon e_mblems"));
   g_object_bind_property (G_OBJECT (dialog->preferences),
                           "tree-icon-emblems",
                           G_OBJECT (button),
@@ -1338,7 +1441,7 @@ thunar_preferences_dialog_init (ThunarPreferencesDialog *dialog)
   gtk_widget_show (grid);
 
   /* add check button to enable/disable auto mounting */
-  button = gtk_check_button_new_with_mnemonic (_("Enable _Volume Management"));
+  button = gtk_check_button_new_with_mnemonic (_("Enable _volume management"));
   g_object_bind_property (G_OBJECT (dialog->preferences),
                           "misc-volume-management",
                           G_OBJECT (button),
