@@ -67,10 +67,10 @@ thunar_progress_view_ask (ThunarProgressView *view,
                           ThunarJobResponse   choices,
                           ThunarJob          *job);
 static ThunarJobResponse
-thunar_progress_view_ask_replace (ThunarProgressView *view,
-                                  ThunarFile         *src_file,
-                                  ThunarFile         *dst_file,
-                                  ThunarJob          *job);
+thunar_progress_view_ask_for_action (ThunarProgressView *view,
+                                     ThunarFile         *src_file,
+                                     ThunarFile         *dst_file,
+                                     ThunarJob          *job);
 static void
 thunar_progress_view_error (ThunarProgressView *view,
                             GError             *error,
@@ -493,10 +493,10 @@ thunar_progress_view_ask (ThunarProgressView *view,
 
 
 static ThunarJobResponse
-thunar_progress_view_ask_replace (ThunarProgressView *view,
-                                  ThunarFile         *src_file,
-                                  ThunarFile         *dst_file,
-                                  ThunarJob          *job)
+thunar_progress_view_ask_for_action (ThunarProgressView *view,
+                                     ThunarFile         *src_file,
+                                     ThunarFile         *dst_file,
+                                     ThunarJob          *job)
 {
   GtkWidget *window;
 
@@ -513,8 +513,8 @@ thunar_progress_view_ask_replace (ThunarProgressView *view,
   window = gtk_widget_get_toplevel (GTK_WIDGET (view));
 
   /* display the question view */
-  return thunar_dialogs_show_job_ask_replace (window != NULL ? GTK_WINDOW (window) : NULL,
-                                              src_file, dst_file, thunar_job_get_n_total_files (job) > 1);
+  return thunar_dialogs_show_job_ask_for_action (window != NULL ? GTK_WINDOW (window) : NULL,
+                                                 src_file, dst_file, thunar_job_get_n_total_files (job) > 1);
 }
 
 
@@ -653,7 +653,10 @@ thunar_progress_view_new_with_job (ThunarJob *job)
   _thunar_return_val_if_fail (job == NULL || THUNAR_IS_JOB (job), NULL);
 
   view = g_object_new (THUNAR_TYPE_PROGRESS_VIEW, "job", job, NULL);
-  gtk_label_set_text (GTK_LABEL (view->progress_label), _("Queued"));
+
+  /* 'thunar_progress_dialog_add_job' will instantly overwrite with "queued", if required */
+  thunar_progress_view_percent (view, 0.0, job);
+
   return GTK_WIDGET (view);
 }
 
@@ -714,7 +717,7 @@ thunar_progress_view_set_job (ThunarProgressView *view,
       g_object_ref (job);
 
       g_signal_connect_swapped (job, "ask", G_CALLBACK (thunar_progress_view_ask), view);
-      g_signal_connect_swapped (job, "ask-replace", G_CALLBACK (thunar_progress_view_ask_replace), view);
+      g_signal_connect_swapped (job, "ask-for-action", G_CALLBACK (thunar_progress_view_ask_for_action), view);
       g_signal_connect_swapped (job, "error", G_CALLBACK (thunar_progress_view_error), view);
       g_signal_connect_swapped (job, "finished", G_CALLBACK (thunar_progress_view_finished), view);
       g_signal_connect_swapped (job, "info-message", G_CALLBACK (thunar_progress_view_info_message), view);
