@@ -15,10 +15,6 @@
  * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
  * Place, Suite 330, Boston, MA  02111-1307  USA
  */
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #include "thunarx/thunarx-menu-item.h"
 #include "thunarx/thunarx-menu.h"
 #include "thunarx/thunarx-private.h"
